      SUBROUTINE PREGION(CGMLT,XLAT,XMLT0,RADIUS,REGB)
!
!  PURPOSE
!     To determine the latitude of the equatorward boundary, maximum
!  and poleward boundary at a given local time given the latitude and
!  local time corresponding to the center of the circular boundary and
!  the radius of this circle.
!
!  METHOD
!    Use the Pythagorean theorem...
!     2  2                       2
!    R +R  - 2.*R R *Cos(Phi) = R
!     0  1       0 1
!
!   Where R = The colatitude of the circle center
!          0
!         R = The circle radius in degrees
!
!       Phi = The current local time - local time of the circle center,
!             in degrees
!
!        R = The colatitude of the boundary at the current local time
!         1
!
!        2                          2   2
!       R  - R *(2.*R *Cos(phi)) + R - R = 0
!        1    1      0              0
! Or
!                               2    2        2   2
!      R = R *Cos(Phi) +- SQRT{R *Cos (Phi) +R - R }
!       1   0                   0                 0
!
!     Choose the postive solution ...in the case of two positive
!   solutions, The smaller one is chosen.
!
!  INPUT PARAMETERS
!    NAME    TYPE       ARRAY  description
!    CGMLT   REAL              The current local time
!    RAD0    REAL       (3)    The radius of the circular boundary
!                              1) Equatorward value
!                              2) Value of maximum
!                              3) Poleward value
!    XLAT    REAL       (3)    The latitude of the center of the circular
!                              boundary
!                              1) Equatorward value
!                              2) Value of maximum
!                              3) Poleward value
!    XMLT0   REAL       (3)    The local time of the center of the circular
!                              boundary
!                              1) Equatorward value
!                              2) Value of maximum
!                              3) Poleward value
!
!  OUTPUT PARAMETERS
!    NAME    TYPE       ARRAY  description
!    REGB    REAL       (3)    The latitude of the circular boundary at local
!                              time CGMLT
!                              1) Equatorward value
!                              2) Value of maximum
!                              3) Poleward value
!
!
!  LOCAL VARIABLES
!    NAME    TYPE       ARRAY  description
!    J       INTEGER           Do loop index
!    SOL1    REAL              One solution of the quadratic equation
!    SOL2    REAL              One solution of the quadratic equation
!    TH1     REAL              The difference between the current local
!                              time and the local time of the center of
!                              the circular  boundary, in degrees, = Phi
!                              in description above.
!    XLT0    REAL              The colatitude of the center of the circ-
!                              ular boundary, in degrees, = R  in description
!                              above.                        0
!
!  SUBROUTINES CALLED
!     NONE
!
!  FUNCTIONS CALLED
!     NONE
!
!  FILES ACCESSED
!     NONE
!
!  AUTHOR
!     William G. Whartenby
!     Computational Physics, Inc.
!     240 Bear Hill Road  Suite 202A
!     Waltham, MA 02154  USA
!     (617)-487-2250
!
!  VERSION
!     1.0.6   01-Dec-1990
!
!  MODIFICATIONS
!     ----Person----  ----Date----  ----------------Description----------------
!
!  REFERENCES
!     NONE
!
!  SPECIAL CONSTANTS
!     EPS = -1.E-3... The maximum a discriminant may go negative
!     and be considered valid (Set equal to 0.0)
!
      REAL EPS
      PARAMETER (EPS = -1.E-3)
      REAL XLAT(3),XMLT0(3),RADIUS(3),REGB(3),DISC,XLT0,TH1,XCT1
      REAL SOL1,SOL2,CGMLT
      INTEGER J
!
!
      DO J = 1,3
!      Compute the discriminant from the equation described above...
!
       XLT0 = 90.-XLAT(J)
       TH1 = (CGMLT-XMLT0(J))*3.1415926/12.
       XCT1 = XLT0*COS(TH1)
       DISC = XCT1**2 - (XLT0**2-RADIUS(J)**2)
!
       IF ( DISC .LT. EPS) THEN
!
!       There is an error
        PRINT *,' Index, lat,local time radius,discriminant = '
        PRINT *,CGMLT,J,XLAT(J),XMLT0(J),RADIUS(J),DISC
        STOP 'NEGATIVE DISCRIMINANT IN PREGION'
!
       ELSE
!
!       Compute the two solutions
!
        IF ( DISC .LT. 0.0) DISC = 0.0
        SOL1 = XCT1 +SQRT(DISC)
        SOL2 = XCT1 -SQRT(DISC)
        IF (MIN(SOL1,SOL2) .GT. 0) THEN
!        Take the minimum
         REGB(J)=MIN(SOL1,SOL2)
!
!        Otherwise , take the postitve one
        ELSE IF (SOL1 .GT. 0) THEN
         REGB(J)=SOL1
        ELSE IF (SOL1 .GT. 0) THEN
         REGB(J)=SOL2
        ELSE
         PRINT *,' Index, lat,local time radius = '
         PRINT *,CGMLT,J,XLAT(J),XMLT0(J),RADIUS(J)
         STOP 'NEGATIVE SOLUTION IN PREGION'
        ENDIF
!       Return the latitude
!
        REGB(J) = 90.-REGB(J)
       ENDIF
      ENDDO
      RETURN
      END
